#!/usr/bin/perl
# THIS PRODUCT CONTAINS RESTRICTED MATERIALS OF IBM
# 5724-X89 (C) COPYRIGHT International Business Machines Corp., 2009
# All Rights Reserved * Licensed Materials - Property of IBM
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
# A sample utility for configuring Linux hostname/ip
#
# version 1.0
###########################################################
use Getopt::Long;
print "Template Activation of Linux Network\n";


#-----------------------------------------------------
# Script for changing the host name and domain used by
#  this virtual image. You can avoid the user input by
#  using command line options.
#-----------------------------------------------------

$CONFIRM_ACTION = 0; # Debug Feature only! Should be 0 otherwise
$VERBOSE = 0;        # Debug Feature only! Should be 0 otherwise.

$HOSTS_FILE= '/etc/hosts';

# Grab the old name & old domain of this image
#
$DEFAULT_XML_FILE;
chomp($OLD_NAME = `hostname`);
chomp($OLD_DOMAIN = `hostname -d`);
if ($VERBOSE){
	print "\n";
	print " Found: Hostname = $OLD_NAME\n";
	print "        Domain = $OLD_DOMAIN\n";
	print "\n";
}


#
# Command line parsing here.
#
if (@ARGV > 0){
	$INTERACTIVE = 0;
	GetOptions( # Debug options
		'verbose' => \$VERBOSE, # Turn on more messages
		'help'    => \$HELP,    # Help
		'debug'   => \$DEBUG,
		#Use xml file as answers to questions
		'xml=s'    => \$DEFAULT_XML_FILE,
		#Command line answers in this section
		'bootproto=s' => \$bootproto,
		'hostname=s'  => \$hostname,
		'domain=s'    => \$domain,
		'ipaddr=s'    => \$ipaddr,
		'netmask=s'   => \$netmask,
		'gateway=s'   => \$gateway,
		'pri_dns=s'   => \$pri_dns,
		'sec_dns=s'   => \$sec_dns,
		'alias=s'	  => \@aliases,
	);

}else{
	$INTERACTIVE = 1;
}


#
# Ask for values, or check they are defined
#
if( $INTERACTIVE){
	
	#
	# Get info by prompting the user
	#
	&question("Hostname",   \$hostname, "Enter the new hostname",   $OLD_NAME );
	&question("Domainname", \$domain,   "Enter the new domainname", $OLD_DOMAIN);
	&question('bootproto'  ,\$bootproto,"Using ('dhcp' or 'static') IP addresses", 'dhcp');
	if ($bootproto =~ /dhcp/i){
		# Skip the ipaddr question with dhcp
	}elsif ($bootproto =~ /static/i){
		&question('ipaddr',  \$ipaddr  , "Enter new ip address", '192.168.0.0');
		&question('gateway', \$gateway , "Enter new gateway",    '192.168.0.1');
		&question('netmask', \$netmask , "Enter new netmask",    '255.255.255.0');
		&question('pri_dns', \$pri_dns , "Enter new Primary DNS",       '192.168.0.2');
		&question('sec_dns', \$sec_dns , "Enter new Secondary DNS",       '192.168.0.3');
	}else{
		die "Don't understand response\n";
	}
	print "\n";
}
else{
	#
	# Command Line Parms supplied: error check that everything specified
	#
	my $err =0;
	die " [-hostname] missing on command line?\n" unless defined $hostname;
	die " [-domain] missing on command line\n" unless defined $domain;
	die " [-bootproto] missing on command line\n" unless defined $bootproto;
	if ($bootproto eq 'static'){
		die "[-ipaddr] missing on command line\n" unless defined $ipaddr;
		die "[-netmask] missing on command line\n" unless defined $netmask;
	}
}

#
# Red Hat specific method using %system-config-network-cmd
#
if (-x '/usr/sbin/system-config-network-cmd'){
	
	print "Configuring Red Hat Linux Network...\n";
	print "Stopping service network...\n";
	print `service network stop`;
	my $NETWORK_CURRENT = '/tmp/network.current';
	my $NETWORK_NEW     = '/tmp/network.new';
	&system_exec("Export the current networking settings",
		"system-config-network-cmd -e > $NETWORK_CURRENT");
	
	#
	# Process the current network settings
	#
	open(INFILE, "<$NETWORK_CURRENT") or die "Can't open $NETWORK_CURRENT\n";
	open(OUTFILE, ">$NETWORK_NEW")    or die "Can't write $NETWORK_NEW\n";
	
	while (<INFILE>){
		chomp $_;
		#print "Line = $_\n";
		if (/(DeviceList.Ethernet.eth0.Hostname)=(.*)/){
			print OUTFILE "${1}=${hostname}\n";
		}elsif (/(DeviceList.Ethernet.eth0.HardwareAddress)=(.*)/){
			# Skip this to enable VM portability
		}elsif (/(DeviceList.Ethernet.eth0.Netmask)=(.*)/){
			# Skip this   -add later if static ip
		}elsif (/(DeviceList.Ethernet.eth0.IP)=(.*)/){
			# Skip this   -add later if static ip
		}elsif (/(DeviceList.Ethernet.eth0.Gateway)=(.*)/){
			# Skip this   -add later if static ip
#ees
                }elsif (/(ProfileList.default.HostsList.*.Hostname)=($OLD_NAME.$OLD_DOMAIN)/){
                        print OUTFILE "${1}=${hostname}.${domain}\n";
                }elsif (/(ProfileList.default.HostsList.*.AliasList.*)=($OLD_NAME.$OLD_DOMAIN)/){
                        print OUTFILE "${1}=${hostname}.${domain}\n";
                }elsif (/(ProfileList.default.HostsList.*.AliasList.*)=($OLD_NAME)/){
                        print OUTFILE "${1}=${hostname}\n";
		}elsif (/(ProfileList.default.DNS.SearchList.1)=(.*)/){
			print OUTFILE "${1}=${domain}\n";
		}elsif (/(ProfileList.default.DNS.Domainname)=(.*)/){
			print OUTFILE "${1}=${domain}\n";
		}elsif (/(ProfileList.default.DNS.Hostname)=(.*)/){
			print OUTFILE "${1}=${hostname}\n";
		}elsif (/(ProfileList.default.DNS.PrimaryDNS)=(.*)/){
			if (defined $pri_dns){
				print OUTFILE "${1}=${pri_dns}\n";
			}else{
				print OUTFILE "${1}=${2}\n";
			}
		}elsif (/(ProfileList.default.DNS.SecondaryDNS)=(.*)/){
			if (defined $pri_dns){
				print OUTFILE "${1}=${sec_dns}\n";
			}else{
				print OUTFILE "${1}=${2}\n";
			}
		}elsif (/(DeviceList.Ethernet.eth0.BootProto)=(.*)/){
			my $BOOTPROTO_LINE = $1;
			if ($bootproto =~ /static/i){
				print OUTFILE "${BOOTPROTO_LINE}=none\n";
				print OUTFILE "DeviceList.Ethernet.eth0.Netmask=$netmask\n";
				print OUTFILE "DeviceList.Ethernet.eth0.IP=$ipaddr\n";
				print OUTFILE "DeviceList.Ethernet.eth0.Gateway=$gateway\n";
			}else{
				print OUTFILE "${BOOTPROTO_LINE}=${bootproto}\n";
			}
		}else{
			# Just print unrecognized lines to the OUTFILE
			#
			print OUTFILE "$_\n"
		}
	}
	close(INFILE);
	close(OUTFILE);
	if ($bootproto eq 'static')
	{
		my $resolv_conf = '/etc/resolv.conf';
		if (-e $resolv_conf){
			if (defined $pri_dns && $pri_dns =~ /\w+/){
				open (FILE, ">$resolv_conf")
					or die "Cannot open $resolv_conf\n";
				print FILE "search $domain\n";
				print FILE "nameserver $pri_dns\n" if defined $pri_dns;
				print FILE "nameserver $sec_dns\n" if defined $sec_dns;
				close (FILE) or die "Can't close file $resolv_conf\n";
			}else{
				`perl -pi -e "s/^search .*/search ${domain}/g;" /etc/resolv.conf`;
			}
		}
	}
#ees
        &system_exec("delete hosts files","rm /etc/hosts /etc/sysconfig/networking/profiles/default/hosts");
        &system_exec("create hosts files","touch /etc/hosts; touch /etc/sysconfig/networking/profiles/default/hosts");
        &system_exec("set permissions hosts files", "chmod 544 /etc/hosts /etc/sysconfig/networking/profiles/default/hosts");	
	open (FILE, ">>$HOSTS_FILE")
		or die "Cannot open $HOSTS_FILE\n";
	foreach my $alias (@aliases){
		print FILE "$alias\n";
	}
	close (FILE) or die "Can't close file $HOSTS_FILE\n";

	&system_exec("Import new networking settings",
		"system-config-network-cmd -i -c -f $NETWORK_NEW");
        &system_exec("delete hosts files","rm /etc/hosts /etc/sysconfig/networking/profiles/default/hosts");
        &system_exec("create hosts files","touch /etc/hosts; touch /etc/sysconfig/networking/profiles/default/hosts");
        &system_exec("set permissions hosts files", "chmod 544 /etc/hosts /etc/sysconfig/networking/profiles/default/hosts");

        &system_exec("Import new networking settings",
                "system-config-network-cmd -i -c -f $NETWORK_NEW");

	&system_exec("Start the network","service network start", "OK");
	&system_exec("Remove temp network files","rm $NETWORK_NEW $NETWORK_CURRENT");
	print "Done configuring Red Hat Linux Network...\n";
	exit(0);
}
elsif (-e '/etc/SuSE-release')
{
	print "Stopping network service...\n";
	print `service network stop`;
	#
	## Step 1. Change the default gateway in the routes file
	#
	if (defined $gateway)
	{
		`echo "default $gateway - -" > /etc/sysconfig/network/routes`;
	}
	# Step 2. Set new hostname
	`export HOSTNAME=$hostname`;
	`hostname $hostname`;
	`echo ${hostname}.${domain} > /etc/HOSTNAME`;
	
	# Step 3. Fix the hosts file with new info
	if (-e $HOSTS_FILE and $bootproto eq 'static')
	{
		`perl -pi -e "s/${OLD_DOMAIN}/${domain}/g;" $HOSTS_FILE`;
		`perl -pi -e "s/.*${OLD_NAME}.*/${ipaddr}\t${hostname}.${domain}\t${hostname}/g;" $HOSTS_FILE`;
	}
	
	open (FILE, ">>$HOSTS_FILE")
		or die "Cannot open $HOSTS_FILE\n";
	foreach my $alias (@aliases)
	{
		print FILE "$alias\n";
	}
	close (FILE) or die "Can't close file $HOSTS_FILE\n";
	
	#
	# Step 1: Find ifcfg-eth*, and modify it.
	#
	# find the actual mac address first
	my $ifresult = `ifconfig -a | grep ^eth | grep HWaddr`;
	$ifresult =~ /.*HWaddr ([0-9a-fA-F:]+)/;
	my $ifconfig_HWaddr = lc($1);
	# find the ifcfg file associated with this ethernet
	my @possible_ifcfg_filename = </etc/sysconfig/network/ifcfg-eth*>;
	if (1 == scalar(@possible_ifcfg_filename))
	{
		my $ifcfg_eth_filename = $possible_ifcfg_filename[0];
		my $ifcfg_eth_newfilename = '';
		if ($ifcfg_eth_filename =~ /$ifconfig_HWaddr/i)
		{
			# The MAC address matches? Maybe detect MAC conflicts here?
			print "ERROR: MAC address doesn't need fixing?\n";
			$ifcfg_eth_newfilename=$ifcfg_eth_filename;
		}
		else
		{
			if ( $ifcfg_eth_filename =~ /(.*)(..:..:..:..:..:..)/ )
			{
				$ifcfg_eth_newfilename = $1 . $ifconfig_HWaddr;
				&system_exec("Move the old $ifcfg_eth_filename to correct $ifconfig_HWaddr",
					"mv $ifcfg_eth_filename $ifcfg_eth_newfilename");
			}
			else
			{
				$ifcfg_eth_newfilename=$ifcfg_eth_filename;
				#die "Don't recognize the $ifcfg_eth_filename?\n";
			}
		}
		open (FILE, ">$ifcfg_eth_newfilename") or die "Cannot open $possible_ifcfg_filename[0]\n";
		print FILE <<"END_FILE_TERMINATOR";
BOOTPROTO='$bootproto'
DHCLIENT_SET_HOSTNAME='$hostname'
BROADCAST=''
ETHTOOL_OPTIONS=''
IPADDR='$ipaddr'
MTU=''
NAME='IBM Virtual Adaptor'
NETMASK='$netmask'
NETWORK=''
REMOTE_IPADDR=''
STARTMODE='auto'
UNIQUE='rBUF.orocOxRg9gF'
USERCONTROL='no'
_nm_name='static-0'
END_FILE_TERMINATOR

	}
	
	#
	# Step 4: Fix the dns name server for static configurations
	#
	if ($bootproto eq 'static')
	{
		my $resolv_conf = '/etc/resolv.conf';
		if (-e $resolv_conf)
		{
			if (defined $pri_dns && $pri_dns =~ /\w+/)
			{
				open (FILE, ">$resolv_conf") or die "Cannot open $resolv_conf\n";
				print FILE "search $domain\n";
				print FILE "nameserver $pri_dns\n" if defined $pri_dns;
				print FILE "nameserver $sec_dns\n" if defined $sec_dns;
				close (FILE) or die "Can't close file $resolv_conf\n";
			}
			else
			{
				`perl -pi -e "s/^search .*/search ${domain}/g;" /etc/resolv.conf`;
			}
		}
	}
	&system_exec("Restart the network","service network start",
		"Setting up service network.*done");
		# allow DHCP to finish before trying to update /etc/hosts
		sleep 10;
	if (-e $HOSTS_FILE and $bootproto eq 'dhcp')
	{
		my $ethname = `ifconfig | grep -i $ifconfig_HWaddr`;
		$ethname =~ s/(eth.).*/$1/;
		chomp($ethname);
		my $newip = `ifconfig $ethname | grep "inet addr"`;
		$newip =~ s|\n$||;
		$newip =~ s|(.*?):(.*?):(.*)|$2|;
		$newip =~ s|(.*?)\s+(.*)|$1|;
		`perl -pi -e "s/${OLD_DOMAIN}/${domain}/g;" $HOSTS_FILE`;
		`perl -pi -e "s/.*${OLD_NAME}.*/${newip}\t${hostname}.${domain}\t${hostname}/g;" $HOSTS_FILE`;
	}
}
else
{
	die "Don't support this Linux version?";
}


#------------------------------------------------
# SUBROUTINE: question                       
#                                                
#  ARG[0]  \$var_ptr     - pointer to set variable
#     [1]  $promptString - what you want to print 
#     [2]  $defaultValue - (optional) a default   
#------------------------------------------------
sub question {
	local($help_topic,$var_ptr,$promptString,$defaultValue) = @_;
	#  If there is a default value, use the first print statement; if
	#  no default is provided, print the second string.
	if ($defaultValue) {
		print $promptString, ": enter[", $defaultValue, "]: ";
	} else {
		print $promptString, ": ";
	}
	
	# We are interactive if the AP file wasn't found & parsed.
	# #
	if ($INTERACTIVE){
		$| = 1;         # force a flush after our print
		$_ = <STDIN>;   # get the input from STDIN (presumably the keyboard)
		chomp;          # remove the newline
		if ("$defaultValue") {
			$$var_ptr = $_ ? $_ : $defaultValue;
		} else {
			$$var_ptr = $_;
		}
	}else{
    # All answers should be filled in already in VAAP file.
		if ($$var_ptr =~ /^\s*$/){
			if (not defined $defaultValue){
				die "ERROR in VAAP file: - missing action for $promptString\n\n";
			}else{
				$$var_ptr = $defaultValue;
				print " ...USE DEFAULT[$$var_ptr]\n";
			}
		}else{
			print "  ...SPECIFIED[$$var_ptr] \n";
		}
	}
	
	if ($$var_ptr =~ /^[Hh]$/ or $$var_ptr =~ /^[Hh]elp$/){
		if (exists $HELP_HASH{$help_topic}){
			print "\nHelp Topic: $help_topic\n";
			print $HELP_HASH{$help_topic};
			print "\n";
		}else{
			print "\nHelp Topic: $help_topic\n";
			print " No help defined for this topic....\n";
			print "\n";
		}
		&question(@_);
	}
}

######################################## 
#     Subroutines                      #
########################################

# system_exec
#   - small utility function for calling a system
#     command and checking for error response.
#
sub system_exec{
	my $description = shift;
	my $cmd = shift;
	my $validate = shift;
	my $res = '';
	
	print "$description\n";
	print " $cmd\n" if $VERBOSE;
	
	if ($CONFIRM_ACTION){
		print "(y|n|a|q) ?";
		my $resp = <>;
		die "Quit response." if $resp =~ /^q/;
		$CONFIRM_ACTION = undef if $resp =~ /^a/;
		return if $resp =~ /^n/;
		die "huh? don't understand $resp" unless $resp =~ /^(y|n|a|q)/;
	}
	
	#
	# Call the system command
	#
	$res = `$cmd`;
	
	#
	# Die on error string found in $?
	#
	die "Failed $description: $? \n$res\n" if ($?);
	
	if (defined $validate){
		if ($res !~ /${validate}/){
			chomp $res;
			print "$res\n";
			die " ERROR: Failed $description -- '$validate'\n";
		}
	}
	return $res;

}
