#!/usr/bin/python

"""
Configure system's hostname and domain 
"""

import os
import re
import sys
from sys import argv

#################
### Functions ###
#################
def _os():
    if os.path.exists('/etc/SuSE-release'):
        type = 'suse'
        ver = ""
        patch = ""
        frls = open('/etc/SuSE-release', 'r')
        for l in frls.readlines():
            v = re.match('VERSION\s=\s(\d*)', l)
            p = re.match('PATCHLEVEL\s=\s(\d*)', l)
            if v != None:
                ver = v.group(1)
            elif p != None:
                patch = p.group(1)
            else:
                pass
        frls.close()
        return {'type': type, 'version': ver + '.' + patch}
    elif os.path.exists('/etc/redhat-release'):
        type = 'redhat'
        frls = open('/etc/redhat-release', 'r')
        rls = frls.readlines()[0]
        version = re.search('.*\s(\d*\.\d*)\s.*', rls).group(1)
        return {'type': type, 'version': version}

def _question(prompt, default):
    s = prompt
    if default != None:
        s = s + "(%s):" % default
    print s
    answer = str(raw_input()).strip()
    if len(answer) == 0:
        return default
    else:
        return answer
    
def _fqdn():
    hostname = os.popen2('hostname')[1].readlines()[0][:-1]
    domain = 'localdomain'
    try:
        domain = os.popen2('hostname --domain')[1].readlines()[0][:-1]
    except:
        pass
    return hostname, domain
    

#################
### Variables ###
#################
files = {}
files['hosts'] = '/etc/hosts'
files['backuphosts'] = '/etc/hosts.ConfigSystem'
files['hostname'] = '/etc/HOSTNAME'

newhostname = None
newdomain = None


##################
#####  Main  #####
##################
if __name__ == '__main__':
    # Read the old FQDN
    oldhostname, olddomain = _fqdn();
    oldfqdn = oldhostname + '.' + olddomain
    print oldhostname 
    print olddomain 
    # Read the arguments
    if len(argv) == 1:
        # No more arguments from CLI, prompt the user
        newhostname = _question("Please input the new hostname", oldhostname)
        newdomain = _question("Please input the new domain name", olddomain)
    else:
        # Read the arguments from CLI
        for i in range(1, len(argv)):
            if argv[i] == '-hostname':
                try:
                    if str(argv[i+1]).startswith('-') == False: newhostname = argv[i+1]
                except:
                    pass
            elif argv[i] == '-domainname':
                try:
                    if str(argv[i+1]).startswith('-') == False: newdomain = argv[i+1]
                except:
                    pass
    
    # Check the arguments
    error = 0
    if newhostname == None:
        print "No new hostname defined."
        error = 1
    if newdomain == None:
        print "No new domain defined."
        error = 1
    if error != 0:
        print "Error(s) arises. Exit."
        sys.exit(error)

    newfqdn = newhostname + '.' + newdomain

    # Update the environment variable
    os.system('export HOSTNAME=%s' % newhostname)
    
    # Invoke hostname command
    os.system('hostname %s' % newhostname)
    
    # Write FQDN to /etc/HOSTNAME
    fhostname = open(files['hostname'], 'wb')
    fhostname.write(newfqdn)
    fhostname.close()
    
    # Update /etc/hosts
    fhosts = open(files['hosts'], 'r')
    oldhosts = fhosts.readlines()
    fhosts.close()
    fhosts = open(files['hosts'], 'wb')
    flag = 0
    if oldhostname == 'Unknown host' :
        for i in range(len(oldhosts)):
            line = str(str(oldhosts[i]).strip())
            if line.find('localhost') > -1:
                flag = 1
            if line.find(oldhostname) > -1:
                flag = 2
                this = line.split();
                ip = this[0]
                for s in this[1:]:
                    if s == oldfqdn: this.remove(s) 
                    elif s == oldhostname: this.remove(s)
                newline = "%s\t%s\t%s\t" % (ip, newfqdn, newhostname)
                if len(this) > 1:
                    newline += "\t".join(this[1:])
                fhosts.write(newline + os.linesep)
            else:
                fhosts.write(line + os.linesep)
    if flag < 1:
        fhosts.write('127.0.0.1 localhost.localdomain localhost' + os.linesep)
    if flag < 2:
        ipaddr = "".join(os.popen('ifconfig eth0 | grep \'inet \' | awk -F \' \' {\'print $2\'} | awk -F \':\' {\'print $2\'}').readlines()).replace("\n", " ")
        fhosts.write('%s %s %s' % (ipaddr, newfqdn, newhostname)  + os.linesep)
    fhosts.close()

    # Add hook script
    ostype = _os().get('type')
    if ostype == 'suse':
        fscript = open('/etc/sysconfig/network/if-up.d/hostname.sh', 'w')
        content = """
#!/bin/sh
FILENAME=$0
if [ $2 = 'eth0' ];
then
    IPADDR=`ifconfig eth0 | grep 'inet ' | cut -d ':' -f 2 | cut -d ' ' -f 1`
    if [ -n "$IPADDR" ] ; then 
        sed -i "s/.*%s/$IPADDR\t%s\t%s/g" /etc/hosts
    fi
    rm -rf $FILENAME
fi
""" % (oldhostname, newfqdn, newhostname)
        fscript.write(content)
        fscript.close()
        os.system('chmod +x /etc/sysconfig/network/if-up.d/hostname.sh')
    elif ostype == 'redhat':
        fscript = open('/sbin/ifup-local', 'w')
        content = """
#!/bin/sh
FILENAME=$0
if [ $1 = 'eth0' ];
then
    IPADDR=`ifconfig eth0 | grep 'inet ' | cut -d ':' -f 2 | cut -d ' ' -f 1`
    sed -i "s/127\.0\.0\.1\t.*/127\.0\.0\.1\tlocalhost\.localdomain\tlocalhost/g" /etc/hosts
    if [ -n "$IPADDR" ] ; then
        sed -i "s/.*%s/$IPADDR\t%s\t%s/g" /etc/hosts
    fi
    sort -u /etc/hosts > /etc/hosts.sorted
    mv /etc/hosts.sorted /etc/hosts
    cp /etc/hosts /etc/sysconfig/networking/profiles/default/hosts
    rm -rf $FILENAME
fi
""" % (oldhostname, newfqdn, newhostname)
        fscript.write(content)
        fscript.close()
        os.system('chmod +x /sbin/ifup-local')
    else:
        pass
    
    # RedHat only
    if ostype == 'redhat':
        netconfs = os.popen2("system-config-network-cmd")[1].readlines()
        ftmpconf = open('/tmp/network.new', 'w')
        for i in range(len(netconfs)):
            line = netconfs[i]
            if re.search('ProfileList\.default\.DNS\.Hostname=.*', line):
                line = 'ProfileList.default.DNS.Hostname=%s' % newhostname
            elif re.search('ProfileList\.default\.DNS\.Domainname=.*', line):
                line = 'ProfileList.default.DNS.Domainname=%s' % newdomain
            else:
                pass
            ftmpconf.write(line + os.linesep)
        ftmpconf.close()
        os.system("system-config-network-cmd -i -c -f /tmp/network.new")
        os.system("/sbin/ifup-local eth0")